from datetime import datetime

from pydantic import Field, model_validator

from . import ApiModel, ItemStatuses, PageInfo, PriorityTypes, UserType
from .account import UserProfile
from .basic import File


class Item(ApiModel):
    id: str = Field(..., alias="id")
    slug: str = Field(..., alias="slug")
    name: str = Field(..., alias="name")
    price: int = Field(..., alias="price")
    raw_price: int = Field(..., alias="rawPrice")

    priority: PriorityTypes | None = Field(None, alias="priority")
    status: ItemStatuses | None = Field(None, alias="status")
    seller_type: UserType | None = Field(None, alias="sellerType")

    attachment: File | None = Field(None, alias="attachment")
    user: UserProfile | None = Field(None, alias="user")

    approval_date: datetime | None = Field(None, alias="approvalDate")
    priority_position: int | None = Field(None, alias="priorityPosition")
    views_counter: int | None = Field(None, alias="viewsCounter")
    fee_multiplier: float | None = Field(None, alias="feeMultiplier")
    created_at: datetime | None = Field(None, alias="createdAt")


class ItemList(ApiModel):
    items: list[Item] = Field(..., alias="items")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "items" not in data:
            edges = data.get("edges") or []
            data["items"] = [(edge or {}).get("node") for edge in edges if (edge or {}).get("node")]

        return data



from pydantic import BaseModel, ConfigDict, Field


class ApiModel(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",
    )


class PageInfo(ApiModel):
    start_cursor: str | None = Field(None, alias="startCursor")
    end_cursor: str | None = Field(None, alias="endCursor")
    has_previous_page: bool | None = Field(None, alias="hasPreviousPage")
    has_next_page: bool | None = Field(None, alias="hasNextPage")


class File(ApiModel):
    id: str = Field(alias="id")
    url: str = Field(alias="url")
    filename: str | None = Field(None, alias="filename")
    mime: str | None = Field(None, alias="mime")



from typing import Any

from .models import (
    GameCategoryDataFieldTypes,
    GameCategoryInstructionTypes,
    GameType,
    ItemsSortOptions,
    QueryID,
)


def _persisted(
    operation_name: str,
    variables: dict[str, Any],
    *,
    sha256_hash: Any | None = None,
    query: str | None = None,
) -> dict[str, Any]:
    r = {
        "operationName": operation_name,
        "variables": variables,
    }
    if query:
        r["query"] = query
    if sha256_hash:
        r["extensions"] = {
            "persistedQuery": {
                "version": 1,
                "sha256Hash": sha256_hash,
            }
        }
    return r


class GraphQLQuery:
    @staticmethod
    def get_me():
        return _persisted(
            operation_name="viewer",
            variables={},
            query="query viewer {\n  viewer {\n    ...Viewer\n    __typename\n  }\n}\n\nfragment Viewer on User {\n  id\n  username\n  email\n  role\n  hasFrozenBalance\n  supportChatId\n  systemChatId\n  unreadChatsCounter\n  isBlocked\n  isBlockedFor\n  isFundsProtectionActive\n  createdAt\n  lastItemCreatedAt\n  hasConfirmedPhoneNumber\n  canPublishItems\n  chosenVerifiedCard {\n    ...MinimalUserBankCard\n    __typename\n  }\n  profile {\n    id\n    avatarURL\n    testimonialCounter\n    __typename\n  }\n  __typename\n}\n\nfragment MinimalUserBankCard on UserBankCard {\n  id\n  cardFirstSix\n  cardLastFour\n  cardType\n  isChosen\n  __typename\n}",
        )

    @staticmethod
    def get_user(username: str | None = None, id: str | None = None) -> dict[str, Any]:
        return _persisted(
            operation_name="user",
            variables={"id": id, "username": username, "hasSupportAccess": False},
            sha256_hash=QueryID.user.value,
        )

    @staticmethod
    def get_games(
        count: int = 24, type: GameType | None = None, cursor: str | None = None
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="games",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {"type": type.name} if type else {},
            },
            sha256_hash=QueryID.games.value,
        )

    @staticmethod
    def get_game(id: str | None = None, slug: str | None = None) -> dict[str, Any]:
        return _persisted(
            operation_name="GamePage",
            variables={"id": id, "slug": slug},
            sha256_hash=QueryID.game.value,
        )

    @staticmethod
    def get_game_category(
        game_id: str | None = None,
        slug: str | None = None,
        id: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="GamePageCategory",
            variables={"id": id, "gameId": game_id, "slug": slug},
            sha256_hash=QueryID.game_category.value,
        )

    @staticmethod
    def get_game_category_agreements(
        game_category_id: str,
        user_id: str | None = None,
        count: int = 24,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryAgreements",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {"gameCategoryId": game_category_id, "userId": user_id},
            },
            sha256_hash=QueryID.game_category_agreements.value,
        )

    @staticmethod
    def get_game_category_obtaining_types(
        game_category_id: str,
        count: int = 24,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryObtainingTypes",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {"gameCategoryId": game_category_id},
            },
            sha256_hash=QueryID.game_category_obtaining_types.value,
        )

    @staticmethod
    def get_game_category_instructions(
        game_category_id: str,
        obtaining_type_id: str,
        count: int = 24,
        type: GameCategoryInstructionTypes | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryInstructions",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {
                    "gameCategoryId": game_category_id,
                    "obtainingTypeId": obtaining_type_id,
                    "type": type.name if type else None,
                },
            },
            sha256_hash=QueryID.game_category_instructions.value,
        )

    @staticmethod
    def get_game_category_data_fields(
        game_category_id: str,
        obtaining_type_id: str,
        count: int = 24,
        type: GameCategoryDataFieldTypes | None = None,
        cursor: str | None = None,
    ) -> dict[str, Any]:
        return _persisted(
            operation_name="gameCategoryDataFields",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": {
                    "gameCategoryId": game_category_id,
                    "obtainingTypeId": obtaining_type_id,
                    "type": type.name if type else None,
                },
            },
            sha256_hash=QueryID.game_category_data_fields.value,
        )

    @staticmethod
    def get_items(
        count: int = 24,
        cursor: str | None = None,
        game_id: str | None = None,
        category_id: str | None = None,
        minimal_price: int | None = None,
        maximal_price: int | None = None,
        has_discount: bool | None = None,
        has_reviews: bool | None = None,
        attributes: list[dict[str, str]] | None = None,
        search: str | None = None,
        sort: ItemsSortOptions | None = None,
    ) -> dict[str, Any]:
        filters = {
            "gameId": game_id,
            "gameCategoryId": category_id,
            "status": ["APPROVED"],
        }
        sort_field = None

        if minimal_price or maximal_price:
            filters["price"] = {"min": minimal_price, "max": maximal_price}
        if has_discount:
            filters["hasDiscount"] = has_discount
        if has_reviews:
            filters["hasTestimonials"] = has_reviews
        if attributes:
            filters["attributes"] = attributes
        if search:
            filters["searchQuery"] = search
        if sort:
            if "PRICE" in sort:
                field = "price"
            else:
                field = "userRating"
            if "ASC" in sort:
                direction = "ASC"
            else:
                direction = "DESC"

            sort_field = {"field": field, "direction": direction}

        return _persisted(
            operation_name="items",
            variables={
                "pagination": {"first": count, "after": cursor},
                "filter": filters,
                "sort": sort_field,
            },
            sha256_hash=QueryID.items.value,
        )

    @staticmethod
    def get_item(id: str | None = None, slug: str | None = None) -> dict[str, Any]:
        return _persisted(
            operation_name="item",
            variables={
                "id": id,
                "slug": slug,
                "hasSupportAccess": False,
                "showForbiddenImage": True,
            },
            sha256_hash=QueryID.item.value,
        )



from ..core import PlayerokClient, _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..models import Item, ItemList, ItemsSortOptions


class ItemsService(PlayerokClient):
    async def get_items(
        self,
        count: int = 24,
        cursor: str | None = None,
        game_id: str | None = None,
        category_id: str | None = None,
        minimal_price: int | None = None,
        maximal_price: int | None = None,
        has_discount: bool | None = None,
        has_reviews: bool | None = None,
        attributes: list[dict[str, str]] | None = None,
        search: str | None = None,
        sort: ItemsSortOptions | None = None,
    ) -> ItemList | None:
        if game_id is None or category_id is None:
            raise ValueError("Can't get items without game_id and category_id")

        response = await self.request(
            "post",
            "graphql",
            GQL.get_items(
                count=count,
                cursor=cursor,
                game_id=game_id,
                category_id=category_id,
                minimal_price=minimal_price,
                maximal_price=maximal_price,
                has_discount=has_discount,
                has_reviews=has_reviews,
                attributes=attributes,
                search=search,
                sort=sort,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "items"))
        if data is None:
            return None

        return ItemList(**data)

    async def get_item(self, id: str | None = None, slug: str | None = None) -> Item | None:
        if id is None and slug is None:
            raise ValueError("Can't get item without id or slug")

        response = await self.request("post", "graphql", GQL.get_item(id=id, slug=slug))
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "item"))
        if data is None:
            return None

        return Item(**data)



from collections.abc import Iterable
from typing import Any


def _dig(obj: dict[str, Any], path: Iterable[str]) -> Any:
    cur: Any = obj
    for key in path:
        if cur is None:
            return None
        cur = cur.get(key)
    return cur


def _raise_on_gql_errors(payload: dict[str, Any]) -> None:
    errors = payload.get("errors")
    if errors:
        raise RuntimeError(f"GraphQL errors: {errors}")



from .basic import ApiModel, PageInfo  # noqa: I001
from .enums import UserType, QueryID  # noqa: I001

from .account import Account, AccountProfile, UserProfile

from .enums import (
    BankCardTypes,
    ChatMessageButtonTypes,
    ChatStatuses,
    ChatTypes,
    EventTypes,
    GameCategoryAgreementIconTypes,
    GameCategoryAutoConfirmPeriods,
    GameCategoryDataFieldInputTypes,
    GameCategoryDataFieldTypes,
    GameCategoryInstructionTypes,
    GameCategoryOptionTypes,
    GameType,
    ItemDealDirections,
    ItemDealStatuses,
    ItemLogEvents,
    ItemsSortOptions,
    ItemStatuses,
    PriorityTypes,
    ReviewStatuses,
    SortDirections,
    TransactionDirections,
    TransactionOperations,
    TransactionPaymentMethodIds,
    TransactionProviderDirections,
    TransactionProviderIds,
    TransactionStatuses,
)
from .games import Game, GameList
from .items import Item, ItemList

__all__ = [
    "Account",
    "UserProfile",
    "AccountProfile",
    "QueryID",
    "GameType",
    "ApiModel",
    "PageInfo",
    "UserType",
    "GameCategoryDataFieldTypes",
    "EventTypes",
    "ItemLogEvents",
    "TransactionOperations",
    "TransactionDirections",
    "TransactionStatuses",
    "TransactionPaymentMethodIds",
    "TransactionProviderDirections",
    "TransactionProviderIds",
    "BankCardTypes",
    "ItemDealStatuses",
    "ItemDealDirections",
    "ChatTypes",
    "ChatStatuses",
    "ChatMessageButtonTypes",
    "ItemStatuses",
    "ReviewStatuses",
    "SortDirections",
    "PriorityTypes",
    "GameCategoryAgreementIconTypes",
    "GameCategoryOptionTypes",
    "GameCategoryDataFieldInputTypes",
    "GameCategoryAutoConfirmPeriods",
    "GameCategoryInstructionTypes",
    "Game",
    "GameList",
    "Item",
    "ItemList",
    "ItemsSortOptions",
]
